# Good ViBrations - a GTK+/GNOME waves simulator
#
# Copyright (C) 2008 Pietro Battiston
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import locale
import gettext
from os.path import exists

APP = 'gvb'
if exists('locale'):
    DIR = 'locale'
else:
    DIR = '/usr/share/locale'

print("Using", DIR)

locale.setlocale(locale.LC_ALL, '')
gettext.bindtextdomain(APP, DIR)
gettext.textdomain(APP)
_ = gettext.gettext



# GVB - a GTK+/GNOME vibrations simulator
#
# Copyright © 2008-2013 Pietro Battiston
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from gi.repository import Gtk, Gdk
from scipy import shape
from time import time
from cairo import ImageSurface, FORMAT_ARGB32, Context

def one_basic(cr, size, pos, speeds = None):
    spaced=1./(len(pos)+1)

    cr.scale(size[0], size[1]/4)
    cr.translate(0, 2)
    cr.set_line_width(.01)
    cr.set_source_rgb(1,0,0)

    cr.move_to(0,0)

    xposition=0
    for ypos in pos:
        xposition=xposition+spaced

        cr.line_to(xposition, -ypos)
    cr.line_to(1,0)
    cr.stroke()


def with_speeds(cr, size, pos, speeds):
    n = len(pos)
    spaced=1./(n+1)
    coeff = .1*n
    cr.scale(size[0], size[1]/4)
    cr.translate(0, 2)
    cr.set_line_width(.1/n)
    cr.set_source_rgb(1,0,0)


    cr.move_to(0,0)

    xposition=spaced
    for idx in range(len(pos)):
        cr.move_to(xposition,-pos[idx])
        cr.rel_line_to(0,-speeds[idx]*coeff)
        cr.stroke()
        xposition=xposition+spaced

    cr.set_line_width(.02)
    cr.set_source_rgb(0,0,1)
    cr.move_to(0,0)

    xposition=0
    for ypos in pos:
        xposition=xposition+spaced

        cr.line_to(xposition, -ypos)
    cr.line_to(1,0)
    cr.stroke()


def temperature(cr, size, pos, speeds = None):
    spaced=1./(len(pos))

    cr.scale(size[0], size[1]/4)
    cr.translate(0, 2)
    cr.set_line_width(.5)
    cr.set_source_rgb(1,0,0)

    cr.move_to(0,0)

    xposition=0
    for ypos in pos:
        cr.move_to(xposition,0)
        xposition=xposition+spaced

        if ypos >= 1:
            cr.set_source_rgb(1,1,1)
        elif ypos > .5:
            cr.set_source_rgb((1-ypos)/2,(1-ypos)/2,0)
        elif ypos > 0:
            cr.set_source_rgb(ypos/2, 1-1.5*ypos, 0)
        elif ypos > -.5:
            cr.set_source_rgb(0,1+2*ypos,-ypos)
        else:
            cr.set_source_rgb(0,0,1+ypos)
        cr.line_to(xposition, 0)
        cr.stroke()


def _two_colours(cr, pos, size, speeds = None):
    hspaced=1./shape(pos)[0]
    vspaced=1./shape(pos)[1]

    cr.scale(size[0], size[1])
    yposition=0
    for ypos in pos:
        xposition=0

        for xpos in ypos:
            if xpos > 0:
                cr.set_source_rgb(1-xpos,1,1)
            else:
                cr.set_source_rgb(1,1+xpos,1)
            cr.rectangle(xposition, yposition, hspaced, vspaced)
            cr.fill()
            xposition=xposition+hspaced
        yposition=yposition+vspaced


def colours(cr, size, pos, speeds = None):
    hspaced=1./shape(pos)[0]
    vspaced=1./shape(pos)[1]

    cr.scale(size[0], size[1])
    cr.translate(0, vspaced/2)
    cr.set_line_width(vspaced)
    yposition=0
    for ypos in pos:
        xposition=0
        cr.move_to(0,yposition)

        for xpos in ypos:
            cr.move_to(xposition, yposition)
            xposition=xposition+hspaced

            if xpos >= 1:
                cr.set_source_rgb(1,1,1)
            elif xpos > .5:
                cr.set_source_rgb((1-xpos)/2,(1-xpos)/2,0)
            elif xpos > 0:
                cr.set_source_rgb(xpos/2, 1-1.5*xpos, 0)
            elif xpos > -.5:
                cr.set_source_rgb(0,1+2*xpos,-xpos)
            else:
                cr.set_source_rgb(0,0,1+xpos)

            cr.line_to(xposition, yposition)
            cr.stroke()
        yposition=yposition+vspaced



def trid(cr, size, pos, speeds = None):
    m,n=shape(pos)

    vspaced=1./(3*n)

    yposition=.3

    cr.set_line_join(2)

    pos = pos * .5

    tr = ([[yposition for i in range(n+2)]]
        +[[yposition+(j+1)*vspaced]+[yposition + (j+1)*vspaced-pos[j][i]/2 for i in range(n)]+[yposition+(j+1)*vspaced] for j in range(m)]
        +[[yposition+(m+1)*vspaced for i in range(n+2)]])

    cr.scale(size[0], size[1])

    cr.set_line_width(.001)

    top_margin = .1

    for row_idx in range(m+1):
        margin = top_margin*(1-(row_idx/m))
        marg_under = top_margin*(1-((row_idx+1)/m))
        hspaced=(1-2*margin)/(n+3)
        xposition = margin + hspaced
        hsp_under=(1-2*marg_under)/(n+3)
        xpos_under = marg_under + hsp_under

        for col_idx in range(n+1):
            if tr[row_idx][col_idx+1] > tr[row_idx+1][col_idx+1]:
                cr.set_source_rgba(1,0,0,.8)
            else:
                cr.set_source_rgba(0,1,0,.8)
            cr.move_to(xposition, tr[row_idx][col_idx])
            cr.line_to(xposition + hspaced, tr[row_idx][col_idx+1])
            cr.line_to(xpos_under + hsp_under, tr[row_idx+1][col_idx+1])
            cr.close_path()
            cr.fill()

            if tr[row_idx][col_idx] > tr[row_idx+1][col_idx]:
                cr.set_source_rgba(1,0,0,.8)
            else:
                cr.set_source_rgba(0,1,0,.8)
            cr.move_to(xposition, tr[row_idx][col_idx])
            cr.line_to(xpos_under + hsp_under, tr[row_idx+1][col_idx+1])
            cr.line_to(xpos_under, tr[row_idx+1][col_idx])
            cr.close_path()
            cr.fill()

            cr.set_source_rgb(0,0,0)

            cr.move_to(xposition, tr[row_idx][col_idx])
            cr.line_to(xposition + hspaced, tr[row_idx][col_idx+1])
            cr.line_to(xpos_under + hsp_under, tr[row_idx+1][col_idx+1])
            cr.line_to(xpos_under, tr[row_idx+1][col_idx])
            cr.close_path()
            cr.stroke()
            xposition=xposition+hspaced
            xpos_under=xpos_under+hsp_under

def trid_speeds(cr, size, pos, speeds = None):
    m,n=shape(pos)

    vspaced=1./(3*n)

    coeff = .05*max(m,n)

    yposition=.3

    cr.set_line_join(2)

    pos = pos * .5

    tr = ([[yposition for i in range(n+2)]]
        +[[yposition+(j+1)*vspaced]+[yposition + (j+1)*vspaced-pos[j][i]/2 for i in range(n)]+[yposition+(j+1)*vspaced] for j in range(m)]
        +[[yposition+(m+1)*vspaced for i in range(n+2)]])

    cr.scale(size[0], size[1])
    cr.set_line_width(.001)

    top_margin = .1

    for row_idx in range(m+1):
        margin = top_margin*(1-(row_idx/m))
        marg_under = top_margin*(1-((row_idx+1)/m))
        hspaced=(1-2*margin)/(n+3)
        xposition = margin + hspaced
        hsp_under=(1-2*marg_under)/(n+3)
        xpos_under = marg_under + hsp_under
#        print ypos
        for col_idx in range(n+1):
            cr.set_line_width(.001)
            if tr[row_idx][col_idx+1] > tr[row_idx+1][col_idx+1]:
                cr.set_source_rgba(0,0,1,.8)
            else:
                cr.set_source_rgba(0,1,0,.8)
            cr.move_to(xposition, tr[row_idx][col_idx])
            cr.line_to(xposition + hspaced, tr[row_idx][col_idx+1])
            cr.line_to(xpos_under + hsp_under, tr[row_idx+1][col_idx+1])
            cr.close_path()
            cr.fill()

            if tr[row_idx][col_idx] > tr[row_idx+1][col_idx]:
                cr.set_source_rgba(0,0,1,.8)
            else:
                cr.set_source_rgba(0,1,0,.8)
            cr.move_to(xposition, tr[row_idx][col_idx])
            cr.line_to(xpos_under + hsp_under, tr[row_idx+1][col_idx+1])
            cr.line_to(xpos_under, tr[row_idx+1][col_idx])
            cr.close_path()
            cr.fill()

            cr.set_source_rgb(0,0,0)

            cr.move_to(xposition, tr[row_idx][col_idx])
            cr.line_to(xposition + hspaced, tr[row_idx][col_idx+1])
            cr.line_to(xpos_under + hsp_under, tr[row_idx+1][col_idx+1])
            cr.line_to(xpos_under, tr[row_idx+1][col_idx])
            cr.close_path()
            cr.stroke()

            if row_idx != 0 and col_idx != 0:
                cr.set_line_width(.1/n)

                cr.set_source_rgba(1,0,0,.5)
                cr.move_to(xposition, tr[row_idx][col_idx])
                cr.rel_line_to(0,-speeds[row_idx-1][col_idx-1]*coeff)
                cr.stroke()

            xposition=xposition+hspaced
            xpos_under=xpos_under+hsp_under

drawers_1d = {
                'wave' :    one_basic,
                'temp' :    temperature,
                'speeds':    with_speeds
            }

drawers_2d = {
                'geo' :        colours,
                '3D' :        trid,
                'speeds':    trid_speeds
            }


drawers_dict={ 1: drawers_1d, 2: drawers_2d }
drawers = {1: ['wave', 'temp', 'speeds'], 2: ['3D', 'geo', 'speeds']}

class Drawer():
    def __init__(self, drawing, combo, dr_type = None):
        self.drawing = drawing
        self.dumpdir = None
        self.combo = combo
        self.active = True
        self.points = None
        self.dim = None
        self.ready = False
        self.drawn = 0
        self.dr_type = dr_type
        self.combo.connect('changed', self.change_drawer)
        if hasattr(self.drawing, 'signals_ids'):
            # This means that the drawing area was already drawn by another
            # drawer. This one is a possessive guy and want her to forget him.
            for signal_id in self.drawing.signals_ids:
                self.drawing.disconnect(signal_id)
        
        # Throw away all his ids! You're mine now!
        self.drawing.signals_ids = []
        self.drawing.signals_ids.append(self.drawing.connect('draw', self.redraw))
        self.size = None

    def draw(self, cr):
        if self.dim != len(self.points.shape):
            raise
        if not self.drawing.get_window():
            return
        if self.dumpdir:
            try:
                cr.set_source_surface(self._last_surface, 0, 0)
            except AttributeError:
                # Still no surface ready
                pass

        before=time()
        if not hasattr(self, 'drawer'):
            return
        size = self.drawing.get_allocated_width(), self.drawing.get_allocated_height()

        cr.rectangle(0, 0, *size)
        cr.set_source_rgb(1, 1, 1)
        cr.fill()
        
        self.drawer(cr, size, self.points.pos, self.points.speeds)
        self.drawn=time()
        self.drawing_time = self.drawn-before

    def dump_draw(self):

        self._last_surface = ImageSurface(FORMAT_ARGB32, self.size[0], self.size[1])

        self.cr = Context(self._last_surface)

        before=time()
        self.drawer(self.cr, self.size, self.points.pos, self.points.speeds)

        self._last_surface.write_to_png('%s/%08d.png' % (self.dumpdir,
                                                         self.dumpindex))
        self.dumpindex += 1
        self.drawn=time()
        self.drawing_time = self.drawn-before
        self.invalidate()
    
    def invalidate(self):
        self.drawing.queue_draw()
    
    def redraw(self, widget, cr):
        self.draw(cr)

    def change_drawer(self, combo):
        self.dr_type = self.combo.get_active_text()
        if self.dr_type in drawers[self.dim]:
            self.drawer=drawers_dict[self.dim][self.dr_type]
            self.drawing.queue_draw()


    def configure(self, points=None, dim=None):
        if not self.active:
            return

        if points:
            self.points = points
            dim = len(shape(points.pos))

        if dim:
            if self.dim != dim:
                self.dim = dim
                self.build_menu()
                self.ready = True
        else:
            return

#        if not (self.points and len(shape(self.points.pos)) == self.dim):    #I won't change dimension if they don't tell me!
#            return

    def build_menu(self):
        combo_model = Gtk.ListStore(str)
        for item in drawers[self.dim]:
            combo_model.append([item])
        self.combo.set_model(combo_model)
        if self.dr_type in drawers[self.dim]:
            self.combo.set_active(drawers[self.dim].index(self.dr_type))
        else:
            self.combo.set_active(0)

    def dump(self, dumpdir):
        print("dumping in", dumpdir)
        self.dumpindex = 0

        if dumpdir:                #This is a "start" command
            self.dumpdir = dumpdir
            self.size = self.drawing.get_allocated_width(), self.drawing.get_allocated_height()
            self.points.changed = self.dump_draw
        else:                        #This is a "stop" command
            self.dumpdir = None
            self.size = None
            self.points.changed = self.invalidate



# GVB - a GTK+/GNOME vibrations simulator
#
# Copyright © 2008-2013 Pietro Battiston
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from scipy import array, zeros, concatenate, shape, split

from .dispositions import disposition
from .calculators import calculators_dict

from time import time

class Points():
    def __init__(self, shape=None, gr=None, disp=None, calc=None, drawer=None, pos=None, from_file=None):
        self.number=0    #Frames numbering
        self.calculating = False

        if from_file:
            self.load(from_file)
        else:
            self.shape=shape

            if pos == None and disp:
                self.pos=disposition(shape, disp)
            else:
                self.pos = pos
                self.disp = 'flat'
            self.speeds=zeros(shape)

        self.gr=gr
        self.drawer = drawer
        self.changed = drawer.invalidate
        drawer.configure(self)
        self.changed()

        self.calc = calc
        if self.calc != None:
            self.calculator=calculators_dict[len(shape)][calc](shape, gr, self.pos, self.speeds)
            self.calc_time = 0
            if self.calculator.discrete:
                self.update=self.update_incremental
            else:
                self.update=self.update_given_time



    def update_incremental(self, given_time):
        calculated=False

        while self.calc_time < given_time:
            before=time()

            self.pos, self.speeds=self.calculator.update(self.pos, self.speeds, given_time)

            last_calc = time()-before

            self.number=self.number+1

            self.calc_time = self.calc_time + self.calculator.gr
            calculated=True

        if calculated:
            self.changed()
            return last_calc*1000
        else:
            return None

    def update_given_time(self, given_time):

        before=time()

        self.pos, self.speeds = self.calculator.update(self.pos, self.speeds, given_time)

        self.last_calc = time() - before

        self.number=self.number+1

        self.calc_time = given_time

        self.changed()

        return self.last_calc*1000
        

    def reconfigure(self, **args):
        if 'pos' in args:
            self.pos=args['pos']
            self.speeds=zeros(self.shape)
            self.calc_time = 0
            self.changed()
        elif 'disp' in args:
            self.disp = args['disp']
            self.pos=disposition(self.shape, self.disp)
            args['pos']=self.pos
            self.speeds=zeros(self.shape)
            self.calc_time = 0
            self.changed()
        elif 'shape' in args:
            self.shape = args['shape']
            self.pos=disposition(self.shape, self.disp)
            self.calc_time = 0
            self.changed()
        if 'calc' in args:
            self.calc = args['calc']
            self.calculator=calculators_dict[len(self.shape)][self.calc](self.shape, self.gr, self.pos, self.speeds)
            self.calc_time = 0
            if self.calculator.discrete:
                self.update=self.update_incremental
            else:
                self.update=self.update_given_time

        elif self.calc:
            self.calculator.reconfigure(**args)

    def dump(self, filename):
        filehand = open(filename, 'w')

        for to_dump in [self.shape, self.pos, self.speeds]:
            while len(shape(to_dump)) > 1:
                to_dump = concatenate(to_dump)
            dump_string = "".join([str(number)+',' for number in to_dump])
            filehand.write(dump_string+'\n')
        filehand.close()

    def load(self, filename):
        filehand = open(filename)

        self.shape = tuple([int(number) for number in filehand.readline()[:-2].split(',')])

        self.pos = array([float(number) for number in filehand.readline()[:-2].split(',')])
        if len(self.shape) == 2:        #FIXME: not ready for dimension > 2 
            self.pos = array(split(self.pos, self.shape[0]))

        self.speeds = array([float(number) for number in filehand.readline()[:-2].split(',')])
        if len(self.shape) == 2:        #FIXME: not ready for dimension > 2 
            self.speeds = array(split(self.speeds, self.shape[0]))




#! /usr/bin/python

#UPDATE:
import os, subprocess

for strings_file in os.listdir('po'):
    if not strings_file.endswith('.po'):
        continue
    lang = strings_file[:-3]
#    subprocess.call(['msgmerge', '-U', 'po/%s' % strings_file, 'po/good-vibrations.pot'])
#    if not os.path.exists('locale/%s/LC_MESSAGES' % lang):
#        os.mkdir('locale/%s' % lang)
#        os.mkdir('locale/%s/LC_MESSAGES' % lang)
    subprocess.call(['msgfmt', 'po/%s' % strings_file, '-o', 'locale/%s/LC_MESSAGES/gvb.mo' % lang])

# (commented because now Launchpad does it)




# GVB - a GTK+/GNOME vibrations simulator
#
# Copyright © 2008-2013 Pietro Battiston
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#Give startup dispositions of the points

from scipy import array, zeros, sin, cos, pi, concatenate, ones, prod, sqrt
from scipy.linalg import norm

from .gvbi18n import _

def disposition(shape, type_descriptor):
    if len(shape) in dispositions and type_descriptor in dispositions[len(shape)]:
        return dispositions_dict[len(shape)][type_descriptor](shape)
    else:
        print("type not found:", type_descriptor)
        return zeros(shape)

waveforms_dict = {    #hl=half wave lenght, cen=centering (from 0 to 1), x is between 1 (not 0) and wl-1
    'square': (lambda hl, ce, t: 1)
    ,
    'triangular': (lambda hl, ce, t: min(t/ce, (hl-t)/(1-ce))/hl )
    ,
    'sinusoidal': (lambda hl, ce, t: sin(min(t/(2*ce), (hl-t)/(2*(1-ce))) * pi/hl) )
    ,
    'peak': (lambda hl, ce, t: max((abs(hl*ce-t),0),(.5,1))[1] )
                }

waveforms = ['sinusoidal', 'triangular', 'square', 'peak']


def waveformer_1d(shape, waveform, lenght, shift, wavelenght, fase_angle, height, centering, rule = None):
    n = shape[0]
    singlewave = array([waveforms_dict[waveform](wavelenght//2+1, centering, t)  for t in range (1,int(wavelenght//2)+1)])
    doublewave = concatenate([singlewave, zeros(1), -1*singlewave])*height

    fase = wavelenght * fase_angle/360
    calculated = doublewave
    while len(calculated) < lenght+fase:
        calculated = concatenate([calculated, array([0]), doublewave])

    calculated = calculated[fase:lenght+fase]

    wave_clip = lenght - max(0, shift + lenght - n)

    zeros_before = shift-(lenght-wave_clip)
    zeros_after = n - (shift + wave_clip)

    wave_final = concatenate([ calculated[wave_clip:], zeros(zeros_before), calculated[:wave_clip], zeros(zeros_after) ])
    return wave_final

def waveformer_2d(shape, *args):
    n1, n2 = shape
    wave1 = waveformer_1d((n1,), *args[:7])
    wave2 = waveformer_1d((n2,), *args[7:14])

    rules = {'sum' : sum, 'prod' : prod, 'max' : max, 'min': min}

    rule = rules[args[14]]

    #FIXME: really not efficient... should someway use scipy.
    wave_final = array( [[rule([wave2[i], wave1[j]]) for i in range(n2)] for j in range(n1)] )

    return wave_final

waveformer = {1: waveformer_1d, 2: waveformer_2d}


dispositions_1d={    #Dispositions are created combining waveformers and/or other dispositions
    'flat': zeros
    ,
#    'sin': (lambda (n,): array([sin(t*pi*2/(n+1)) for t in range(1,n+1)]))
    'sin': (lambda s: waveformer_1d(s, 'sinusoidal', s[0], 0, s[0], 0, 1, .5 ) )
    ,
#    'half sin': (lambda (n,): array([sin(t*pi/(n+1)) for t in range(1,n+1)]))
    'half sin': (lambda s: waveformer_1d(s, 'sinusoidal', s[0], 0, 2*s[0], 0, 1, .5 ) )
    ,
#    'picked': (lambda (n,): array([float(2*min(t, n-t+1))/n for t in range(1,n+1) ]))
    'picked': (lambda s: waveformer_1d(s, 'triangular', s[0], 0, 2*s[0], 0, 1, .5 ) )
    ,
    'triangular signal': (lambda s: concatenate([ disposition((s[0]//4,), 'picked'), disposition((s[0]-s[0]//4,), 'flat') ]))
    ,
    'sinusoidal signal': (lambda s: concatenate([ disposition((s[0]//4,), 'sin'), disposition((s[0]-s[0]//4,), 'flat') ]))
    ,
#    'picked lateral':  (lambda (n,): array([float(min(float(2*t)/n, float(n-t+1)/n)) for t in range(1,n+1) ]))
    'picked lateral': (lambda s: waveformer_1d(s, 'triangular', s[0], 0, 2*s[0], 0, 1, .25 ) )
    ,
    'opposite triangulars': (lambda s: concatenate([ disposition((s[0]//4,), 'picked'), disposition((s[0]-s[0]//4-s[0]//4,), 'flat'), disposition((s[0]//4,), 'picked') ]))
    ,
    'opposite sinusoidals': (lambda s: concatenate([ disposition((s[0]//4,), 'sin'), disposition((s[0]-s[0]//4-s[0]//4,), 'flat'), disposition((s[0]//4,), 'sin') ]))
    ,
    'square': ones
    ,
    'square signal': (lambda s: concatenate([ disposition((s[0]//4,), 'square'), disposition((s[0]-s[0]//4,), 'flat') ]))
    ,
#    'discontinuous peak': (lambda (n,): concatenate([ disposition((n/2,), 'flat'), [1], disposition((n-n/2-1,), 'flat') ]))
    'discontinuous peak': (lambda s: waveformer_1d(s, 'peak', s[0], 0, 2*s[0], 0, 1, .5 ) )
    ,
    'cos (shifted)': (lambda s: waveformer_1d(s, 'sinusoidal', s[0], 0, s[0], 270, 1, .5) + ones(s) )
                }


dispositions_2d={
    'flat': zeros
    ,
#    'picked': (lambda (n,m): array([array([float(2*min(t, m-t+1))/n for t in range(1,m+1) ])*float(2*min(j, n-j+1))/n for j in range(1,n+1)]))
    'picked': (lambda s: waveformer_2d(s, 'triangular', s[0], 0, 2*s[0], 0, 1, .5, 'triangular', s[1], 0, 2*s[1], 0, 1, .5 , 'prod' ) )
    ,
    'sin': (lambda s: waveformer_2d(s, 'sinusoidal', s[0], 0, s[0], 0, 1, .5, 'sinusoidal', s[1], 0, s[1], 0, 1, .5 , 'prod' ) )
    ,
    'half sin': (lambda s: waveformer_2d(s, 'sinusoidal', s[0], 0, 2*s[0], 0, 1, .5, 'sinusoidal', s[1], 0, 2*s[1], 0, 1, .5 , 'prod' ) )
    ,
    'pond': (lambda s: array([[(lambda x,y,r : -.1*cos(10*pi*norm([x,y])/r) * max(1-norm([x,y])/r, 0) ) (i-s[1]/2, j-s[0]/2, min(*s))/2 for i in range(1,s[1]+1) ] for j in range(1,s[0]+1)]))
    ,
    'waterfall': (lambda s: array([[(lambda x,y : 1 if norm([x,y]) < min(*s)/10 else 0)(i-s[1]/2, j-s[0]/2) for i in range(1,s[1]+1) ] for j in range(1,s[0]+1)]))
    ,
    'gut': (lambda s: array([[(lambda x,y,r : -1-cos(8*pi*norm([x,y])/r) if norm([x,y]) < r/8 else 0) (i-s[1]/2, j-s[0]/2, min(*s)) for i in range(1,s[1]+1) ] for j in range(1,s[0]+1)]))
                }

dispositions_dict={1:dispositions_1d, 2:dispositions_2d}

dispositions={}


for dim in [1,2]:
    keys = list(dispositions_dict[dim].keys())
    keys.sort()
    dispositions[dim]=keys






dummy_list_for_gettext=[_('flat'),    _('sin'), _('half sin'), _('picked'), _('triangular signal'), _('sinusoidal signal'), _('picked lateral'), _('opposite triangulars'), _('picked'), _('opposite sinusoidals'), _('square'), _('square signal'), _('discontinuous peak'), _('cos (shifted)'), _('pond'), _('waterfall'), _('gut'), _('peak'), _('sinusoidal') ]



# GVB - a GTK+/GNOME vibrations simulator
#
# Copyright © 2008-2013 Pietro Battiston
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from gi.repository import Gtk
from gi.repository import GObject

from gvbmod import drawers, dispositions, points

from scipy import array     #Just until I find a "revert order" method for arrays...

param_names = ['lenght', 'shift', 'wavelenght', 'fase', 'amplitude', 'centering']
int_params = ['lenght', 'shift', 'wavelenght', 'fase']
rule_names = ['prod', 'sum', 'max', 'min']

reflect = -1
label = -2
rules = -3
rules_hbox = -4
drawer = -5

DIMS = [1,2]

def reflect_matrix(matrix):
    n = matrix.shape[0]
    return array([matrix[n - 1 - i ] for i in range(n) ])

class AdvancedEditor():
    def __init__(self, caller, xml, old_points, shape):
        self.caller = caller
        self.dim = len(shape)
        self.points = None
        self.ready = False
        self.skipped_one = False
        self.xml = xml

        self.window = xml.get_object('advanced editor')
        self.for_good = True

        self.widgets = {}
        for dim in DIMS:
            self.widgets[dim] = {}
            self.widgets[dim]['waveform'] = xml.get_object('editor waveform combo '+str(dim))

            self.combo_model=Gtk.ListStore(str,str)
            for waveform in dispositions.waveforms:

                self.combo_model.append([_(waveform), waveform])
            self.widgets[dim]['waveform'].set_model(self.combo_model)
            self.widgets[dim]['waveform'].set_active(0)

            self.widgets[dim][reflect] = xml.get_object('editor reflect check '+str(dim))
            self.widgets[dim][label] = xml.get_object('editor label '+str(dim))

            for name in param_names:
                self.widgets[dim][name] = xml.get_object('editor '+name+' scale '+str(dim))

        self.widgets[drawer] = xml.get_object('editor combo drawer')
        self.widgets[rules]=[]
        self.widgets[rules_hbox] = xml.get_object('editor rules hbox')

        for rule in rule_names:
            self.widgets[rules].append(xml.get_object('editor rule '+rule))


        self.invert_check = xml.get_object('editor invert check')
        self.add_check = xml.get_object('editor addto check')
        
        for dim in DIMS:
            for widget in self.widgets[dim]:
                if isinstance(self.widgets[dim][widget], Gtk.ComboBox):
                    self.widgets[dim][widget].connect('changed', self.update)
                elif isinstance(self.widgets[dim][widget], Gtk.CheckButton):
                    self.widgets[dim][widget].connect('toggled', self.update)
                elif isinstance(self.widgets[dim][widget], Gtk.HScale):
                    self.widgets[dim][widget].connect('change-value', self.update)
        
        for widget in  [self.invert_check, self.add_check] + self.widgets[rules]:
            widget.connect('toggled', self.update)
        
        self.go(old_points, shape)


    def go(self, old_points, shape):            

        self.old_points = old_points
        self.shape = shape
        self.dim = len(shape)

#        self.drawer=drawers.drawers[self.dim][self.widgets[drawer].get_active_text()]

        for dim in range(1, self.dim+1):
            if self.points == None or len(self.points.shape) < dim or self.points.shape[dim-1] != shape[dim-1]:
                try:
                    size = shape[dim-1]
                except IndexError:
                    size = shape[0]
                print("dim", dim, "size", size)
                #If number of points or dimension changed since last editor call (or if this is the first call)...
                self.widgets[dim]['lenght'].set_adjustment(Gtk.Adjustment(size, 3, size, 4, 10))
                self.widgets[dim]['shift'].set_adjustment(Gtk.Adjustment(0, 0, size, 4, 10))
                self.widgets[dim]['wavelenght'].set_adjustment(Gtk.Adjustment(size, 5, size*3, 4, 10))

        for dim in DIMS:
            useful = not (dim > self.dim)
            for widget in self.widgets[dim]:
                self.widgets[dim][widget].set_sensitive(useful)

        self.widgets[rules_hbox].set_sensitive(bool(self.dim-1))

        #If dimension is changing with this editor call, we cannot add
        self.add_check.set_sensitive(old_points.shape == shape)
        
        self.points = points.Points(self.shape, None, 'flat', None, drawers.Drawer(self.xml.get_object('editor drawing'), self.xml.get_object('editor combo drawer')))

        GObject.timeout_add(10, self.update)

        resp = self.window.run()
        while True:
            self.response(None, resp)
            if resp == 2:
                resp = self.window.run()
            else:
                break


    def update(self, arg=None, arg2=None, arg3=None):
        if arg in self.widgets[rules]:        #The radiobuttons always send 2 signals: one for the toggled and one for the untoggled
            if self.skipped_one:
                self.skipped_one = False
            else:
                self.skipped_one = True
                return

        params = []
        for dim in range(1, self.dim+1):
            selected_iter = self.widgets[dim]['waveform'].get_active_iter()
            liststore = self.widgets[dim]['waveform'].get_model()
            params=params + [liststore.get(selected_iter, 1)[0]]
            params_dim = [self.widgets[dim][widget].get_value() for widget in param_names]
            for index in range(len(int_params)):
                params_dim[index]=int(params_dim[index])

            params = params + params_dim

        #Get selected rule:
        params.append(rule_names[ [rule.get_active() for rule in self.widgets[rules]].index(True)]) #FIXME (ugly)
        self.new_disp = dispositions.waveformer[self.dim](self.shape, *params)


        if self.dim == 1:
            if self.widgets[1][reflect].get_active():
                self.new_disp = reflect_matrix(self.new_disp)
        else:
            if self.widgets[2][reflect].get_active():
                self.new_disp = reflect_matrix(self.new_disp)
            if self.widgets[1][reflect].get_active():
                self.new_disp = reflect_matrix(self.new_disp.transpose()).transpose()


        if self.invert_check.get_active():
            self.new_disp = self.new_disp * -1
        if self.add_check.get_active():
            self.points.reconfigure(pos = self.old_points.pos + self.new_disp)
        else:
            self.points.reconfigure(pos = self.new_disp)
        self.ready = True
        self.points.drawer.invalidate()


    def set_defaults(self):
        defaults = [self.shape[0], 0, self.shape[1], 0, 1, .5]

        for dim in range(1, self.dim+1):
            self.widgets[dim]['waveform'].set_active(0)
            for index in range(len(param_names)):
                self.widgets[dim][param_names[index]].set_value(defaults[index])
            self.widgets[dim][reflect].set_active(False)

        self.widgets[rules][0].set_active(True)
        self.invert_check.set_active(False)
        self.add_check.set_active(False)
        self.update()


    def response(self, button, response):
        if response == 2:
            self.set_defaults()
            return

        if response == 1:
            self.caller.editor_save(self.points)

        self.window.hide()



# GVB - a GTK+/GNOME vibrations simulator
#
# Copyright © 2008-2013 Pietro Battiston
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from scipy import array, dot, diag, identity, bmat, concatenate, exp, transpose, zeros, real, sum, ones, split, sqrt, sin, arccos, sign, repeat, matrix
from scipy.linalg import eig, inv, solve

import time as timer


class Calculator():
    def __init__(self, n, gr, pos, speeds):
        self.n=n
        self.gr=gr
        self.start=pos
        self.startspeeds=speeds

    def reconfigure(self, **args):
        if 'shape' in args:
            self.n=args['shape']
        if 'gr' in args:
            self.gr=args['gr']
        if 'pos' in args:
            self.start=args['pos']

class Calculator1d(Calculator):
    '''
    Overriding two methods just because I want "self.n" to be an int.
    '''
    def __init__(self, *args):
        Calculator.__init__(self, *args)
        self.n=self.n[0]

    def reconfigure(self, **args):
        Calculator.reconfigure(self, **args)
        if 'shape' in args and args['shape']:
            self.n=self.n[0]

class Calculator_naif(Calculator1d):
    discrete=True
    def update(self, points, speeds, time):
        accels=array([2*points[0]-points[1]]+[2*points[i]-points[i-1]-points[i+1] for i in range(1,self.n-1)]+[2*points[self.n-1]-points[self.n-2]])
        new_speeds=speeds-dot(self.gr, accels)
        new_points=points+dot(self.gr, new_speeds)

        return new_points, new_speeds

class Calculator_quad(Calculator1d):
    discrete=True
    def update(self, points, speeds, time):
        step=1/(self.n+1)
        qstep=step**2
        astep = repeat(step,self.n+1)
        aqstep = repeat(qstep,self.n+1)

        factor = (lambda ip : ip * sin(arccos(astep/ip)) )

        strenght = (lambda ydiff : factor(sqrt(ydiff*ydiff+aqstep)) * sign(ydiff) )

        strenghts = strenght(concatenate([points, zeros(1)]) - concatenate([zeros(1), points]))

        new_speeds=speeds+self.gr*(strenghts[1:]-strenghts[:-1])
        new_points=points+dot(self.gr, new_speeds)

        return new_points, new_speeds


class Calculator_matrixnaif(Calculator1d):
    discrete=True
    def __init__(self, *args):
        Calculator1d.__init__(self, *args)

        self.build_mat()

    def build_mat(self):
        n_id=identity(self.n)
        self.op=diag([-2]*self.n)+diag([1]*(self.n-1),1)+diag([1]*(self.n-1),-1)
        self.mat=array(bmat([[n_id, self.gr*n_id], [self.gr*self.op, n_id]]))

    def reconfigure(self, **args):
        Calculator1d.reconfigure(self, **args)
        self.build_mat()

    def update(self, points, speeds, time):
        new_coords=dot(self.mat, concatenate([points, speeds]))

        return new_coords[:self.n], new_coords[self.n:]

class Calculator_eig(Calculator1d):
    discrete=False
    def __init__(self, *args):
        Calculator1d.__init__(self, *args)

        self.build_mat()
        self.build_operator()
        self.time=0
        

    def build_mat(self):#FIXME:merge 
        n_id=identity(self.n)
        n_zero=zeros([self.n, self.n])
        self.op=diag([-2]*self.n)+diag([1]*(self.n-1),1)+diag([1]*(self.n-1),-1)
        self.mat=array(bmat([[n_zero, n_id], [self.op, n_zero]]))
        self.eigval, self.eigvec = eig(self.mat)

    def build_operator(self):
        self.startvec = solve(self.eigvec, concatenate([self.start, self.startspeeds]))
        self.operator=dot(self.eigvec, diag(self.startvec)).transpose()

    def reconfigure(self, **args): 
        Calculator1d.reconfigure(self, **args)
        if 'shape' in args:
            self.build_mat()
            self.build_operator()
            return
        if 'pos' in args:
            self.build_operator()

    def update(self, points, speeds, time):
        expmultiplier=exp(self.eigval*time)
        result=real(dot(expmultiplier, self.operator))    #This dot product is perfectly equivalent to mutliplying by a diagonal and then adding rows

        return result[:self.n], result[self.n:]



class Calculator_naif2d(Calculator):
    discrete=True
    def update(self, points, speeds, time):
        n=self.n[0]
        m=self.n[1]
        accels=array(
                    [
                    [ 4*points[0][0]-points[1][0]-points[0][1] ]
                    +[ 4*points[0][i]-points[0][i-1]-points[0][i+1]-points[1][i] for i in range(1,n-1) ]
                    +[ 4*points[0][n-1]-points[0][n-2]-points[1][n-1] ]
                    ]
                +
                    [
                    [ 4*points[j][0]-points[j+1][0]-points[j-1][0]-points[j][1] ]
                    +[ 4*points[j][i]-points[j][i-1]-points[j][i+1]-points[j+1][i]-points[j-1][i] for i in range(1,n-1) ]
                    +[ 4*points[j][n-1]-points[j][n-2]-points[j+1][n-1]-points[j-1][n-1] ]
                    for j in range(1, m-1)]
                +
                    [
                    [ 4*points[m-1][0]-points[n-2][0]-points[n-1][1] ]
                    +[ 4*points[m-1][i]-points[m-1][i-1]-points[m-1][i+1]-points[m-2][i] for i in range(1,n-1)]
                    +[ 4*points[m-1][n-1]-points[m-1][n-2]-points[m-2][n-1] ]
                    ]
                    )

        new_speeds=speeds-dot(self.gr, accels)
        new_points=points+dot(self.gr, new_speeds)

        return new_points, new_speeds

class Calculator_matrixnaif2d(Calculator):
    discrete=True
    def __init__(self, *args):
        Calculator.__init__(self, *args)

        self.build_mat()

    def build_mat(self):
        self.edge = self.n[0]*self.n[1]
        small_id = identity(self.n[1])
        small_zeros = zeros((self.n[1], self.n[1]))
        n_id = identity(self.edge)
        small_op = diag(ones(self.n[1]-1),1) - 4*identity(self.n[1]) + diag(ones(self.n[1]-1),-1)
#        self.op=diag([-2]*self.n)+diag([1]*(self.n-1),1)+diag([1]*(self.n-1),-1)
        #rowlenght:
        rl = self.n[1]

        self.op = bmat([
                        [small_op, small_id] + [small_zeros for i in range(2, self.n[0])]
                        ]+[
                        [small_zeros for i in range(j)] + [small_id, small_op, small_id] + [small_zeros for i in range(j+1, rl-2)]
                        for j in range(rl-2)] + [
                        [small_zeros for i in range(2, self.n[0])] + [small_id, small_op]
                        ])

        self.mat=array(bmat([[n_id, self.gr*n_id], [self.gr*self.op, n_id]]))

    def reconfigure(self, **args):
        Calculator.reconfigure(self, **args)
        self.build_mat()

    def update(self, points_matrix, speeds_matrix, time):
        points = concatenate( points_matrix)
        speeds = concatenate(speeds_matrix)

        new_coords=array(split(dot(self.mat, concatenate([points, speeds])), 2*self.n[0]))

        return new_coords[:self.n[0]], new_coords[self.n[0]:]

class Calculator_eig2d(Calculator):
    discrete=False
    def __init__(self, *args):
        Calculator.__init__(self, *args)

        self.build_mat()
        self.build_operator()
        self.time=0
        

    def build_mat(self):#FIXME:merge 
        self.edge = self.n[0]*self.n[1]
        small_id = identity(self.n[1])
        small_zeros = zeros((self.n[1], self.n[1]))
        n_id = identity(self.edge)
        n_zero = zeros([self.edge, self.edge])
        small_op = diag(ones(self.n[1]-1),1) - 4*identity(self.n[1]) + diag(ones(self.n[1]-1),-1)
        # Number of blocks (per dimension):
        rl = self.n[0]
        self.op = bmat([
                        [small_op, small_id] + [small_zeros for i in range(2, rl)]
                        ]+[
                        [small_zeros for i in range(j)] + [small_id, small_op, small_id] + [small_zeros for i in range(j+1, rl-2)]
                        for j in range(rl-2)] + [
                        [small_zeros for i in range(rl-2)] + [small_id, small_op]
                        ])
        self.mat=array(bmat([[n_zero, n_id], [self.op, n_zero]]))
        self.eigval, self.eigvec = eig(self.mat)

    def build_operator(self):
        self.startvec = solve(self.eigvec, concatenate([concatenate(self.start), concatenate(self.startspeeds)]))
        self.operator=dot(self.eigvec, diag(self.startvec)).transpose()

    def reconfigure(self, **args): 
        Calculator.reconfigure(self, **args)
        if 'shape' in args:
            self.build_mat()
            self.build_operator()
            return
        if 'pos' in args:
#            print "rebuilding"
            self.build_operator()

    def update(self, points, speeds, time):
        expmultiplier=exp(self.eigval*time)
        result=array(split(real(dot(expmultiplier, self.operator)), self.n[0]*2))    #This dot product is perfectly equivalent to mutliplying by a diagonal and then adding rows

        return result[:self.n[0]], result[self.n[0]:]

class Calculator_quad_2D(Calculator):
    discrete=True
    def __init__(self, *args):
        Calculator.__init__(self, *args)

        self.build_lambdas()
        self.time=0

    def build_lambdas(self):
        hstep=1/(self.n[1]+1)
        qhstep=hstep**2
        mhstep = array(repeat(matrix(repeat(hstep,self.n[1]+1)), self.n[0], 0))
        mqhstep = array(repeat(matrix(repeat(qhstep,self.n[1]+1)), self.n[0], 0))
        vstep=1/(self.n[0]+1)
        qvstep=vstep**2
        mvstep = array(repeat(matrix(repeat(vstep,self.n[1])), self.n[0]+1, 0))
        mqvstep = array(repeat(matrix(repeat(qvstep,self.n[1])), self.n[0]+1, 0))

        hfactor = (lambda ip : ip * sin(arccos(mhstep/ip)) )
        self.hstrenght = (lambda zdiff : hfactor(sqrt(zdiff*zdiff+mqhstep)) * sign(zdiff) )

        vfactor = (lambda ip : ip * sin(arccos(mvstep/ip)) )
        self.vstrenght = (lambda zdiff : vfactor(sqrt(zdiff*zdiff+mqvstep)) * sign(zdiff) )

    def reconfigure(self, **args): 
        Calculator.reconfigure(self, **args)
        if 'shape' in args:
            self.build_lambdas()

    def update(self, pos, speeds, time):

        hstrenghts = self.hstrenght(concatenate([pos, zeros([self.n[0],1])], 1 ) - concatenate([zeros([self.n[0],1]), pos],1 ))
        vstrenghts = self.vstrenght(concatenate([pos, [zeros(self.n[1])]]) - concatenate([[zeros(self.n[1])], pos]))

        new_speeds=speeds+self.gr*(hstrenghts[:,1:]-hstrenghts[:,:-1] + vstrenghts[1:]-vstrenghts[:-1])
        new_points=pos+self.gr*new_speeds

        return new_points, new_speeds



calculators_1d={'eig': Calculator_eig, 'naif': Calculator_naif, 'matrix': Calculator_matrixnaif, 'quad': Calculator_quad}

calculators_2d={'eig': Calculator_eig2d, 'naif': Calculator_naif2d, 'matrix': Calculator_matrixnaif2d,  'quad': Calculator_quad_2D}#, 'eig': Calculator_eig2}

calculators_dict={1: calculators_1d, 2: calculators_2d}

calculators={
            1: ['eig', 'quad', 'matrix', 'naif'],
            2: ['eig', 'quad', 'matrix', 'naif']
            }



#!/usr/bin/python3

from os import listdir, path
from distutils.core import setup
from DistUtilsExtra.command import *
import sys
import subprocess

dist = setup(name='gvb',
      version='1.4',
      description='Good ViBrations',
      license='GPL',
      author='Pietro Battiston',
      author_email='me@pietrobattiston.it',
      url='http://www.pietrobattiston.it/gvb',
      scripts=['gvb'],
      packages=['gvbmod'],
      data_files=[('share/gvb/stuff', ['stuff/logo.svg', 'stuff/wlogo.svg', 'stuff/gvb.glade']),
#                  ('share/doc/gvb', ['README']),
                  ('share/pixmaps', ['stuff/gvb.svg']),
                  ('share/applications', ['stuff/gvb.desktop'])]+
                  [('share/locale/'+lang+'/LC_MESSAGES/', ['locale/'+lang+'/LC_MESSAGES/gvb.mo'] ) for lang in listdir('locale')],
      cmdclass = { "build" :  build_extra.build_extra,
                   "build_help" :  build_help.build_help},
      classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: X11 Applications :: GTK',
        'Intended Audience :: Education',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python',
        'Topic :: Education :: Testing',
        'Topic :: Scientific/Engineering :: Physics',
        'Topic :: Scientific/Engineering :: Visualization',
        ]    
     )



#! /usr/bin/python

import subprocess, os

#PREPARE THE TEMPLATE:
subprocess.call(['xgettext', '-o', 'po/good-vibrations.pot', 'gvb.py', 'gvbmod/dispositions.py', 'stuff/gvb.glade'])

#INITIALIZE:
#msginit --input=messages-all.pot
#mv it.po po/it.po
#mkdir -p trunk/locale/it/LC_MESSAGES/

#UPDATE:

#for strings_file in os.listdir('po'):
#    if not strings_file.endswith('.po'):
#        continue
#    lang = strings_file[:-3]
#    subprocess.call(['msgmerge', '-U', 'po/%s' % strings_file, 'po/good-vibrations.pot'])
#    if not os.path.exists('locale/%s/LC_MESSAGES' % lang):
#        os.mkdir('locale/%s' % lang)
#        os.mkdir('locale/%s/LC_MESSAGES' % lang)
#    subprocess.call(['msgfmt', 'po/%s' % strings_file, '-o', 'locale/%s/LC_MESSAGES/gvb.mo' % lang])

# (commented because now Launchpad does it)
